/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.dynamic.data.mapping.form.values.query.internal;

import com.liferay.dynamic.data.mapping.form.values.query.DDMFormValuesQuery;
import com.liferay.dynamic.data.mapping.form.values.query.internal.model.DDMFormValuesFilter;
import com.liferay.dynamic.data.mapping.storage.DDMFormFieldValue;
import com.liferay.dynamic.data.mapping.storage.DDMFormValues;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Adolfo Pérez
 * @author Marcellus Tavares
 */
public class DDMFormValuesQueryImpl implements DDMFormValuesQuery {

	public DDMFormValuesQueryImpl(
		DDMFormValues ddmFormValues,
		List<DDMFormValuesFilter> ddmFormFieldValueFilters) {

		_ddmFormValues = ddmFormValues;
		_ddmFormFieldValueFilters = ddmFormFieldValueFilters;
	}

	@Override
	public List<DDMFormFieldValue> selectDDMFormFieldValues() {
		DDMFormValuesFilter firstDDMFormValuesFilter =
			_ddmFormFieldValueFilters.get(0);

		List<DDMFormFieldValue> ddmFormFieldValues =
			firstDDMFormValuesFilter.filter(_ddmFormValues);

		for (int i = 1; i < _ddmFormFieldValueFilters.size(); i++) {
			DDMFormValuesFilter currentDDMFormValuesFilter =
				_ddmFormFieldValueFilters.get(i);

			List<DDMFormFieldValue> nestedDDMFormFieldValues =
				_getNestedDDMFormFieldValues(ddmFormFieldValues);

			ddmFormFieldValues = currentDDMFormValuesFilter.filter(
				nestedDDMFormFieldValues);
		}

		return ddmFormFieldValues;
	}

	@Override
	public DDMFormFieldValue selectSingleDDMFormFieldValue() {
		List<DDMFormFieldValue> ddmFormFieldValues = selectDDMFormFieldValues();

		if (ddmFormFieldValues.isEmpty()) {
			return null;
		}

		return ddmFormFieldValues.get(0);
	}

	private List<DDMFormFieldValue> _getNestedDDMFormFieldValues(
		List<DDMFormFieldValue> ddmFormFieldValues) {

		List<DDMFormFieldValue> nestedDDMFormFieldValues = new ArrayList<>();

		for (DDMFormFieldValue ddmFormFieldValue : ddmFormFieldValues) {
			nestedDDMFormFieldValues.addAll(
				ddmFormFieldValue.getNestedDDMFormFieldValues());
		}

		return nestedDDMFormFieldValues;
	}

	private final List<DDMFormValuesFilter> _ddmFormFieldValueFilters;
	private final DDMFormValues _ddmFormValues;

}